#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
from PyQt5.QtWidgets import *
from PyQt5.QtGui import QColor, QPalette, QFont
from PyQt5.QtCore import pyqtSignal, QObject, Qt, QCoreApplication
import cruscoplanets.cmdline


class FontLineEdit(QWidget):

	fontChanged = pyqtSignal(str)

	def __init__(self, value):
		super().__init__()
		self.lineEdit = QLineEdit(value)
		self.lineEdit.textChanged.connect(self._handleTextChanged)
		self.button = QPushButton("...")
		self.button.clicked.connect(self._openFontDialogue)
		layout = QHBoxLayout()
		layout.setContentsMargins(0, 0, 0, 0)
		layout.addWidget(self.lineEdit)
		layout.addWidget(self.button)
		self.setLayout(layout)
	
	def _openFontDialogue(self):
		start_font = self.lineEdit.text()
		new_font, ok = QFontDialog().getFont(QFont(start_font, 10), self)
		if ok:
			new_font = new_font.toString().split(',')[0]#font family name
		else:
			new_font = start_font
		self.lineEdit.setText(new_font)
		
	def _handleTextChanged(self, text):
		self.fontChanged.emit(text)


class ColorLineEdit(QWidget):

	colorChanged = pyqtSignal(str)

	def __init__(self, value):
		super().__init__()
		self.lineEdit = QLineEdit(value)
		self._changeLineEditColor(QColor(value))
		self.__currentColor = value
		self.lineEdit.textChanged.connect(self._handleTextChanged)
		self.button = QPushButton("...")
		self.button.clicked.connect(self._openColorDialogue)
		layout = QHBoxLayout()
		layout.setContentsMargins(0, 0, 0, 0)
		layout.addWidget(self.lineEdit)
		layout.addWidget(self.button)
		self.setLayout(layout)

	def _changeLineEditColor(self, color):
		linePalette = QPalette()
		linePalette.setColor(QPalette.Base, color)
		if color == Qt.black:
			linePalette.setColor(QPalette.Text, QColor("white"))
		else:
			linePalette.setColor(QPalette.Text, QColor("black"))
		self.lineEdit.setPalette(linePalette)

	
	def _openColorDialogue(self):
		start_color = QColor(self.lineEdit.text()) if QColor(self.lineEdit.text()).isValid() else "#000000"
		new_color = QColorDialog().getColor(initial=start_color)
		if new_color.isValid():
			new_color = new_color.name()
		else:
			new_color = self.__currentColor
		self.lineEdit.setText(new_color)
		
	def _handleTextChanged(self, text):
		if QColor(text).isValid():
			self.__currentColor = text
			self._changeLineEditColor(QColor(text))
			self.colorChanged.emit(text)


class ScaleSlider(QSlider):

	valueSet = pyqtSignal(float)

	def __init__(self, value):
		super().__init__(Qt.Horizontal)
		self.setMinimum(0)
		self.setMaximum(100)
		self.setSingleStep(1)
		self.setValue(value*10)
		self.valueChanged.connect(lambda: self.valueSet.emit(self.value()/10))


class PercentageSlider(QSlider):

	valueSet = pyqtSignal(float)

	def __init__(self, value):
		super().__init__(Qt.Horizontal)
		self.setMinimum(0)
		self.setMaximum(100)
		self.setSingleStep(1)
		self.setValue(value*100)
		self.valueChanged.connect(lambda: self.valueSet.emit(self.value()/100))


class FontWeightComboBox(QComboBox):

	valueSet = pyqtSignal(str)

	VALUES = ("lighter", "normal", "bolder", "bold")	
	
	def __init__(self, myvalue):
		super().__init__()
		self.addItem(self.tr("lighter"))
		self.addItem(self.tr("normal"))
		self.addItem(self.tr("bolder"))
		self.addItem(self.tr("bold"))
		myindex = self.__class__.VALUES.index(myvalue)
		self.setCurrentIndex(myindex)
		self.activated.connect(self._onActivated)
		
	def _onActivated(self, index):
		self.valueSet.emit(self.__class__.VALUES[index])


class FontStyleComboBox(QComboBox):

	valueSet = pyqtSignal(str)

	VALUES = ("normal", "italic", "oblique")	
	
	def __init__(self, myvalue):
		super().__init__()
		self.addItem(self.tr("normal"))
		self.addItem(self.tr("italic"))
		self.addItem(self.tr("oblique"))
		myindex = self.__class__.VALUES.index(myvalue)
		self.setCurrentIndex(myindex)
		self.activated.connect(self._onActivated)
		
	def _onActivated(self, index):
		self.valueSet.emit(self.__class__.VALUES[index])


class ViewBoxSpin(QDoubleSpinBox):

	SINGLE_STEP = 10

	def __init__(self, minimum, maximum):
		super().__init__()
		self.setRange(minimum, maximum)
		self.setSingleStep(self.__class__.SINGLE_STEP)
		
	def keyPressEvent(self, event):
		event.ignore()
		
		
class ViewBoxSetter(QWidget):

	valueSet = pyqtSignal(str)

	def __init__(self, value):
		super().__init__()
		self._ratio = None#if set to None, width and height can change independently. If set to a float value, they will scale according to that ratio
		values = tuple(float(x) for x in value.split())
		center_x, center_y, self.width, self.height = values
		self.center_x, self.center_y = -center_x, -center_y

		self.center_x_spin = ViewBoxSpin(minimum = -self.width, maximum = self.width)
		self.center_x_spin.setValue(self.center_x)
		self.center_y_spin = ViewBoxSpin(minimum = -self.height, maximum = self.height)
		self.center_y_spin.setValue(self.center_y)
		self.width_spin = ViewBoxSpin(minimum = 0, maximum = 10*self.width)
		self.width_spin.setValue(self.width)
		self.width_spin.valueChanged.connect(self._controlHeight)
		self.width_spin.valueChanged.connect(lambda value: self._adjustCenter(self.center_x_spin, value))
		self.height_spin = ViewBoxSpin(minimum = 0, maximum = 10*self.height)
		self.height_spin.setValue(self.height)
		self.height_spin.valueChanged.connect(self._controlWidth)
		self.height_spin.valueChanged.connect(lambda value: self._adjustCenter(self.center_y_spin, value))

		self.keepRatio = QCheckBox(self.tr("keep_image_ratio"))
		self.keepRatio.stateChanged.connect(self._onKeepRatioChecked)

		self.width_spin.valueChanged.connect(lambda: self.valueSet.emit(self.valuesString()))
		self.height_spin.valueChanged.connect(lambda: self.valueSet.emit(self.valuesString()))
		self.center_x_spin.valueChanged.connect(lambda: self.valueSet.emit(self.valuesString()))
		self.center_y_spin.valueChanged.connect(lambda: self.valueSet.emit(self.valuesString()))
		
	def _controlHeight(self, width):
		if self._ratio != None:
			height = width/self._ratio
			self.height_spin.setValue(height)
		
	def _controlWidth(self, height):
		if self._ratio != None:
			width = height*self._ratio
			self.width_spin.setValue(width)

	def _onKeepRatioChecked(self):
		if self.keepRatio.isChecked():
			self._ratio = self.width/self.height
		else:
			self._ratio = None
		
	def _adjustCenter(self, widget, value):
		widget.setMaximum(value)
		widget.setMinimum(-value)

	def rowsSequence(self):
		return (
			(self.tr("CENTRE_X"), self.center_x_spin),
			(self.tr("CENTRE_Y"), self.center_y_spin),
			(self.tr("WIDTH"), self.width_spin),
			(self.tr("HEIGHT"), self.height_spin),
			("", self.keepRatio,)
		)
		
	def valuesString(self):
		return "%.1f %.1f %.1f %.1f"%(-self.center_x_spin.value(), -self.center_y_spin.value(), self.width_spin.value(), self.height_spin.value())

		
class SettingsOption(QObject):

	optionChanged = pyqtSignal([str, str], [str, float])
	COLOR = 0
	SIZE = 1
	PERCENTAGE = 2
	FONT_WEIGHT = 3
	FONT_STYLE = 4
	IMAGE_VIEWBOX = 5
	FONT_FAMILY = 6
	SCALE = 7
	
	def __init__(self, key, value, translation, opt_type):
		super().__init__()
		self.key = key
		self.translation = translation
		self.type = opt_type
		self.widget = self._buildWidget(value)
		
	def _buildWidget(self, myvalue):
		if self.type == self.__class__.SIZE:
			widget = QDoubleSpinBox(singleStep=1.0, maximum=10*myvalue, minimum=0)
			widget.setValue(myvalue)
			widget.valueChanged.connect(lambda value: self.optionChanged[str, float].emit(self.key, value))
		elif self.type == self.__class__.COLOR:
			widget = ColorLineEdit(myvalue)
			widget.colorChanged.connect(lambda value: self.optionChanged[str, str].emit(self.key, value))
		elif self.type == self.__class__.PERCENTAGE:
			widget = PercentageSlider(myvalue)
			widget.valueSet.connect(lambda value: self.optionChanged[str, float].emit(self.key, value))
		elif self.type == self.__class__.FONT_WEIGHT:
			widget = FontWeightComboBox(myvalue)
			widget.valueSet.connect(lambda value: self.optionChanged[str, str].emit(self.key, value))
		elif self.type == self.__class__.FONT_STYLE:
			widget = FontStyleComboBox(myvalue)
			widget.valueSet.connect(lambda value: self.optionChanged[str, str].emit(self.key, value))
		elif self.type == self.__class__.IMAGE_VIEWBOX:
			widget = ViewBoxSetter(myvalue)
			widget.valueSet.connect(lambda value: self.optionChanged[str, str].emit(self.key, value))
		elif self.type == self.__class__.FONT_FAMILY:
			widget = FontLineEdit(myvalue)
			widget.fontChanged.connect(lambda value: self.optionChanged[str, str].emit(self.key, value))
		elif self.type == self.__class__.SCALE:
			widget = ScaleSlider(myvalue)
			widget.valueSet.connect(lambda value: self.optionChanged[str, float].emit(self.key, value))
			
		return widget


class SettingsFormTab(QScrollArea):

	optionsChanged = pyqtSignal([str, str], [str, float])

	def __init__(self, options_tuple):
		super().__init__()
		self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.settingsClass = cruscoplanets.cmdline.svgSettings()
		self.settingsOptions = tuple(SettingsOption(option[0], self.settingsClass[option[0]], option[1], option[2]) for option in options_tuple)
		self.container = QWidget()
		layout = QFormLayout()
		for settings_option in self.settingsOptions:
			if settings_option.type != SettingsOption.IMAGE_VIEWBOX:
				layout.addRow(settings_option.translation, settings_option.widget)
			else:
				for row in settings_option.widget.rowsSequence():
					layout.addRow(*row)
			settings_option.optionChanged[str, str].connect(self._updateSettings)
			settings_option.optionChanged[str, float].connect(self._updateSettings)
		self.container.setLayout(layout)
		self.setWidget(self.container)

	def _updateSettings(self, key, value):
		self.settingsClass[key] = value
		self.optionsChanged[str, type(value)].emit(key, value)


class OptionsList(QObject):#unfortunately it can not be set as singleton, but anyway I will istantiate it just once

	def __init__(self):
		super().__init__()
		self.FONT_OPTIONS = (
			("FONT_FAMILY", self.tr("FONT_FAMILY"), SettingsOption.FONT_FAMILY),
			("PERSONAL_DATA_FONT_COLOR", self.tr("PERSONAL_DATA_FONT_COLOR"), SettingsOption.COLOR),
			("PERSONAL_DATA_FONT_WEIGHT", self.tr("PERSONAL_DATA_FONT_WEIGHT"), SettingsOption.FONT_WEIGHT),
			("PERSONAL_DATA_FONT_STYLE", self.tr("PERSONAL_DATA_FONT_STYLE"), SettingsOption.FONT_STYLE),
			("PERSONAL_DATA_FONT_SIZE", self.tr("PERSONAL_DATA_FONT_SIZE"), SettingsOption.SIZE),
			("PLANETS_TABLE_FONT_COLOR", self.tr("PLANETS_TABLE_FONT_COLOR"), SettingsOption.COLOR),
			("PLANETS_TABLE_FONT_WEIGHT", self.tr("PLANETS_TABLE_FONT_WEIGHT"), SettingsOption.FONT_WEIGHT),
			("PLANETS_TABLE_FONT_STYLE", self.tr("PLANETS_TABLE_FONT_STYLE"), SettingsOption.FONT_STYLE),
			("PLANETS_TABLE_FONT_SIZE", self.tr("PLANETS_TABLE_FONT_SIZE"), SettingsOption.SIZE),
			("HOUSES_TABLE_FONT_COLOR", self.tr("HOUSES_TABLE_FONT_COLOR"), SettingsOption.COLOR),
			("HOUSES_TABLE_FONT_WEIGHT", self.tr("HOUSES_TABLE_FONT_WEIGHT"), SettingsOption.FONT_WEIGHT),
			("HOUSES_TABLE_FONT_STYLE", self.tr("HOUSES_TABLE_FONT_STYLE"), SettingsOption.FONT_STYLE),
			("HOUSES_TABLE_FONT_SIZE", self.tr("HOUSES_TABLE_FONT_SIZE"), SettingsOption.SIZE),
			("TRANSITS_TABLE_FONT_COLOR", self.tr("TRANSITS_TABLE_FONT_COLOR"), SettingsOption.COLOR),
			("TRANSITS_TABLE_FONT_WEIGHT", self.tr("TRANSITS_TABLE_FONT_WEIGHT"), SettingsOption.FONT_WEIGHT),
			("TRANSITS_TABLE_FONT_STYLE", self.tr("TRANSITS_TABLE_FONT_STYLE"), SettingsOption.FONT_STYLE),
			("TRANSITS_TABLE_FONT_SIZE", self.tr("TRANSITS_TABLE_FONT_SIZE"), SettingsOption.SIZE),
			("ASPECTS_GRID_TITLE_FONT_WEIGHT", self.tr("ASPECTS_GRID_TITLE_FONT_WEIGHT"), SettingsOption.FONT_WEIGHT),
			("ASPECTS_GRID_TITLE_FONT_STYLE", self.tr("ASPECTS_GRID_TITLE_FONT_STYLE"), SettingsOption.FONT_STYLE),
			("ASPECTS_GRID_TITLE_FONT_SIZE", self.tr("ASPECTS_GRID_TITLE_FONT_SIZE"), SettingsOption.SIZE)
		)

		self.LAYOUT_OPTIONS = (
			("IMAGE_VIEWBOX", self.tr("IMAGE_VIEWBOX"), SettingsOption.IMAGE_VIEWBOX),
			("IMAGE_STROKE", self.tr("IMAGE_STROKE"), SettingsOption.COLOR),
			("BACKGROUND_COLOR", self.tr("BACKGROUND_COLOR"), SettingsOption.COLOR),
			("ASPECTS_GRID_STROKE_OPACITY", self.tr("ASPECTS_GRID_STROKE_OPACITY"), SettingsOption.PERCENTAGE),
			("ASPECTS_GRID_FILL_OPACITY", self.tr("ASPECTS_GRID_FILL_OPACITY"), SettingsOption.PERCENTAGE),
			("PERSONAL_DATA_X_OFFSET", self.tr("PERSONAL_DATA_X_OFFSET"), SettingsOption.SIZE),
			("PERSONAL_DATA_Y_OFFSET", self.tr("PERSONAL_DATA_Y_OFFSET"), SettingsOption.SIZE),
			("ASPECTS_GRID_STROKE_COLOR", self.tr("ASPECTS_GRID_STROKE_COLOR"), SettingsOption.COLOR),
			("ASPECTS_GRID_FILL_COLOR", self.tr("ASPECTS_GRID_FILL_COLOR"), SettingsOption.COLOR),
			("PLANETS_TABLE_X_OFFSET", self.tr("PLANETS_TABLE_X_OFFSET"), SettingsOption.SIZE),
			("PLANETS_TABLE_Y_OFFSET", self.tr("PLANETS_TABLE_Y_OFFSET"), SettingsOption.SIZE),
			("PLANETS_TABLE_CELL_LENGTH", self.tr("PLANETS_TABLE_CELL_LENGTH"), SettingsOption.SIZE),
			("HOUSES_TABLE_X_OFFSET", self.tr("HOUSES_TABLE_X_OFFSET"), SettingsOption.SIZE),
			("HOUSES_TABLE_Y_OFFSET", self.tr("HOUSES_TABLE_Y_OFFSET"), SettingsOption.SIZE),
			("HOUSES_TABLE_CELL_LENGTH", self.tr("HOUSES_TABLE_CELL_LENGTH"), SettingsOption.SIZE),
			("TRANSITS_TABLE_X_OFFSET", self.tr("TRANSITS_TABLE_X_OFFSET"), SettingsOption.SIZE),
			("TRANSITS_TABLE_Y_OFFSET", self.tr("TRANSITS_TABLE_Y_OFFSET"), SettingsOption.SIZE),
			("TRANSITS_TABLE_CELL_LENGTH", self.tr("TRANSITS_TABLE_CELL_LENGTH"), SettingsOption.SIZE),
			("ASPECTS_GRID_X_OFFSET", self.tr("ASPECTS_GRID_X_OFFSET"), SettingsOption.SIZE),
			("ASPECTS_GRID_Y_OFFSET", self.tr("ASPECTS_GRID_Y_OFFSET"), SettingsOption.SIZE),
			("ASPECTS_GRID_CELL_LENGTH", self.tr("ASPECTS_GRID_CELL_LENGTH"), SettingsOption.SIZE),
			("ASPECTS_GRID_STROKE_WIDTH", self.tr("ASPECTS_GRID_STROKE_WIDTH"), SettingsOption.SIZE),
		)

		self.SYMBOLS_COLORS = (
			("CHAR-ARIES_SIGN_COLOR", self.tr("CHAR-ARIES_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-TAURUS_SIGN_COLOR", self.tr("CHAR-TAURUS_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-GEMINI_SIGN_COLOR", self.tr("CHAR-GEMINI_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-CANCER_SIGN_COLOR", self.tr("CHAR-CANCER_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-LEO_SIGN_COLOR", self.tr("CHAR-LEO_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-VIRGO_SIGN_COLOR", self.tr("CHAR-VIRGO_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-LIBRA_SIGN_COLOR", self.tr("CHAR-LIBRA_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-SCORPIO_SIGN_COLOR", self.tr("CHAR-SCORPIO_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-SAGITTARIUS_SIGN_COLOR", self.tr("CHAR-SAGITTARIUS_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-CAPRICORN_SIGN_COLOR", self.tr("CHAR-CAPRICORN_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-AQUARIUS_SIGN_COLOR", self.tr("CHAR-AQUARIUS_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-PISCES_SIGN_COLOR", self.tr("CHAR-PISCES_SIGN_COLOR"), SettingsOption.COLOR),
			("CHAR-SUN_COLOR", self.tr("CHAR-SUN_COLOR"), SettingsOption.COLOR),
			("CHAR-MOON_COLOR", self.tr("CHAR-MOON_COLOR"), SettingsOption.COLOR),
			("CHAR-MERCURY_COLOR", self.tr("CHAR-MERCURY_COLOR"), SettingsOption.COLOR),
			("CHAR-VENUS_COLOR", self.tr("CHAR-VENUS_COLOR"), SettingsOption.COLOR),
			("CHAR-MARS_COLOR", self.tr("CHAR-MARS_COLOR"), SettingsOption.COLOR),
			("CHAR-JUPITER_COLOR", self.tr("CHAR-JUPITER_COLOR"), SettingsOption.COLOR),
			("CHAR-SATURN_COLOR", self.tr("CHAR-SATURN_COLOR"), SettingsOption.COLOR),
			("CHAR-URANUS_COLOR", self.tr("CHAR-URANUS_COLOR"), SettingsOption.COLOR),
			("CHAR-NEPTUNE_COLOR", self.tr("CHAR-NEPTUNE_COLOR"), SettingsOption.COLOR),
			("CHAR-PLUTO_COLOR", self.tr("CHAR-PLUTO_COLOR"), SettingsOption.COLOR),
			("CHAR-ERIS_COLOR", self.tr("CHAR-ERIS_COLOR"), SettingsOption.COLOR),
			("ARIES_COLOR", self.tr("ARIES_COLOR"), SettingsOption.COLOR),
			("TAURUS_COLOR", self.tr("TAURUS_COLOR"), SettingsOption.COLOR),
			("GEMINI_COLOR", self.tr("GEMINI_COLOR"), SettingsOption.COLOR),
			("CANCER_COLOR", self.tr("CANCER_COLOR"), SettingsOption.COLOR),
			("LEO_COLOR", self.tr("LEO_COLOR"), SettingsOption.COLOR),
			("VIRGO_COLOR", self.tr("VIRGO_COLOR"), SettingsOption.COLOR),
			("LIBRA_COLOR", self.tr("LIBRA_COLOR"), SettingsOption.COLOR),
			("SCORPIO_COLOR", self.tr("SCORPIO_COLOR"), SettingsOption.COLOR),
			("SAGITTARIUS_COLOR", self.tr("SAGITTARIUS_COLOR"), SettingsOption.COLOR),
			("CAPRICORN_COLOR", self.tr("CAPRICORN_COLOR"), SettingsOption.COLOR),
			("AQUARIUS_COLOR", self.tr("AQUARIUS_COLOR"), SettingsOption.COLOR),
			("PISCES_COLOR", self.tr("PISCES_COLOR"), SettingsOption.COLOR),
			("ARIES_SIGN_COLOR", self.tr("ARIES_SIGN_COLOR"), SettingsOption.COLOR),
			("TAURUS_SIGN_COLOR", self.tr("TAURUS_SIGN_COLOR"), SettingsOption.COLOR),
			("GEMINI_SIGN_COLOR", self.tr("GEMINI_SIGN_COLOR"), SettingsOption.COLOR),
			("CANCER_SIGN_COLOR", self.tr("CANCER_SIGN_COLOR"), SettingsOption.COLOR),
			("LEO_SIGN_COLOR", self.tr("LEO_SIGN_COLOR"), SettingsOption.COLOR),
			("VIRGO_SIGN_COLOR", self.tr("VIRGO_SIGN_COLOR"), SettingsOption.COLOR),
			("LIBRA_SIGN_COLOR", self.tr("LIBRA_SIGN_COLOR"), SettingsOption.COLOR),
			("SCORPIO_SIGN_COLOR", self.tr("SCORPIO_SIGN_COLOR"), SettingsOption.COLOR),
			("SAGITTARIUS_SIGN_COLOR", self.tr("SAGITTARIUS_SIGN_COLOR"), SettingsOption.COLOR),
			("CAPRICORN_SIGN_COLOR", self.tr("CAPRICORN_SIGN_COLOR"), SettingsOption.COLOR),
			("AQUARIUS_SIGN_COLOR", self.tr("AQUARIUS_SIGN_COLOR"), SettingsOption.COLOR),
			("PISCES_SIGN_COLOR", self.tr("PISCES_SIGN_COLOR"), SettingsOption.COLOR),
			("SUN_COLOR", self.tr("SUN_COLOR"), SettingsOption.COLOR),
			("MOON_COLOR", self.tr("MOON_COLOR"), SettingsOption.COLOR),
			("MERCURY_COLOR", self.tr("MERCURY_COLOR"), SettingsOption.COLOR),
			("VENUS_COLOR", self.tr("VENUS_COLOR"), SettingsOption.COLOR),
			("MARS_COLOR", self.tr("MARS_COLOR"), SettingsOption.COLOR),
			("JUPITER_COLOR", self.tr("JUPITER_COLOR"), SettingsOption.COLOR),
			("SATURN_COLOR", self.tr("SATURN_COLOR"), SettingsOption.COLOR),
			("URANUS_COLOR", self.tr("URANUS_COLOR"), SettingsOption.COLOR),
			("NEPTUNE_COLOR", self.tr("NEPTUNE_COLOR"), SettingsOption.COLOR),
			("PLUTO_COLOR", self.tr("PLUTO_COLOR"), SettingsOption.COLOR),
			("ERIS_COLOR", self.tr("ERIS_COLOR"), SettingsOption.COLOR),
			("TRANSIT-SUN_COLOR", self.tr("TRANSIT-SUN_COLOR"), SettingsOption.COLOR),
			("TRANSIT-MOON_COLOR", self.tr("TRANSIT-MOON_COLOR"), SettingsOption.COLOR),
			("TRANSIT-MERCURY_COLOR", self.tr("TRANSIT-MERCURY_COLOR"), SettingsOption.COLOR),
			("TRANSIT-VENUS_COLOR", self.tr("TRANSIT-VENUS_COLOR"), SettingsOption.COLOR),
			("TRANSIT-MARS_COLOR", self.tr("TRANSIT-MARS_COLOR"), SettingsOption.COLOR),
			("TRANSIT-JUPITER_COLOR", self.tr("TRANSIT-JUPITER_COLOR"), SettingsOption.COLOR),
			("TRANSIT-SATURN_COLOR", self.tr("TRANSIT-SATURN_COLOR"), SettingsOption.COLOR),
			("TRANSIT-URANUS_COLOR", self.tr("TRANSIT-URANUS_COLOR"), SettingsOption.COLOR),
			("TRANSIT-NEPTUNE_COLOR", self.tr("TRANSIT-NEPTUNE_COLOR"), SettingsOption.COLOR),
			("TRANSIT-PLUTO_COLOR", self.tr("TRANSIT-PLUTO_COLOR"), SettingsOption.COLOR),
			("TRANSIT-ERIS_COLOR", self.tr("TRANSIT-ERIS_COLOR"), SettingsOption.COLOR),
			("CONJUNCTION_COLOR", self.tr("CONJUNCTION_COLOR"), SettingsOption.COLOR),
			("OPPOSITION_COLOR", self.tr("OPPOSITION_COLOR"), SettingsOption.COLOR),
			("TRINE_COLOR", self.tr("TRINE_COLOR"), SettingsOption.COLOR),
			("SQUARE_COLOR", self.tr("SQUARE_COLOR"), SettingsOption.COLOR),
			("SEXTILE_COLOR", self.tr("SEXTILE_COLOR"), SettingsOption.COLOR),
			("SEMISEXTILE_COLOR", self.tr("SEMISEXTILE_COLOR"), SettingsOption.COLOR),
		)
		
		self.ZODIAC_OPTIONS = (
			("OUTER_BORDER_COLOR", self.tr("OUTER_BORDER_COLOR"), SettingsOption.COLOR),
			("OUTER_BORDER_WIDTH", self.tr("OUTER_BORDER_WIDTH"), SettingsOption.SIZE),
			("SIGN_BORDERS_COLOR", self.tr("SIGN_BORDERS_COLOR"), SettingsOption.COLOR),
			("SIGN_BORDERS_WIDTH", self.tr("SIGN_BORDERS_WIDTH"), SettingsOption.SIZE),
			("ZODIAC_CIRCLE_RADIUS", self.tr("ZODIAC_CIRCLE_RADIUS"), SettingsOption.SIZE),
			("HOUSES_CIRCLE_RADIUS", self.tr("HOUSES_CIRCLE_RADIUS"), SettingsOption.SIZE),
			("ASPECTS_CIRCLE_RADIUS", self.tr("ASPECTS_CIRCLE_RADIUS"), SettingsOption.SIZE),
			("ZODIAC_OPACITY", self.tr("ZODIAC_OPACITY"), SettingsOption.PERCENTAGE),
			("SIGN_SCALE", self.tr("SIGN_SCALE"), SettingsOption.SCALE),
			("PLANET_SCALE", self.tr("PLANET_SCALE"), SettingsOption.SCALE),
			("DEGREES_CIRCLE_STROKE_OPACITY", self.tr("DEGREES_CIRCLE_STROKE_OPACITY"), SettingsOption.PERCENTAGE),
			("DEGREES_CIRCLE_FILL_OPACITY", self.tr("DEGREES_CIRCLE_FILL_OPACITY"), SettingsOption.PERCENTAGE),
			("ASPECTS_CIRCLE_STROKE_OPACITY", self.tr("ASPECTS_CIRCLE_STROKE_OPACITY"), SettingsOption.PERCENTAGE),
			("ASPECTS_CIRCLE_FILL_OPACITY", self.tr("ASPECTS_CIRCLE_FILL_OPACITY"), SettingsOption.PERCENTAGE),
			("HOUSE_SYMBOLS_COLOR", self.tr("HOUSE_SYMBOLS_COLOR"), SettingsOption.COLOR),
			("HOUSE_SYMBOLS_SIZE", self.tr("HOUSE_SYMBOLS_SIZE"), SettingsOption.SIZE),
			("HOUSE_SYMBOLS_OFFSET", self.tr("HOUSE_SYMBOLS_OFFSET"), SettingsOption.SIZE),
			("DEGREES_CIRCLE_COLOR", self.tr("DEGREES_CIRCLE_COLOR"), SettingsOption.COLOR),
			("TENTH_DEGREE_LINE", self.tr("TENTH_DEGREE_LINE"), SettingsOption.SIZE),
			("FIFTH_DEGREE_LINE", self.tr("FIFTH_DEGREE_LINE"), SettingsOption.SIZE),
			("DEGREE_LINE", self.tr("DEGREE_LINE"), SettingsOption.SIZE),
			("ASPECTS_CIRCLE_COLOR", self.tr("ASPECTS_CIRCLE_COLOR"), SettingsOption.COLOR),
			("DEGREES_CIRCLE_FILL_COLOR", self.tr("DEGREES_CIRCLE_FILL_COLOR"), SettingsOption.COLOR),
			("ASPECTS_CIRCLE_FILL_COLOR", self.tr("ASPECTS_CIRCLE_FILL_COLOR"), SettingsOption.COLOR),
			("HOUSE_LINE_COLOR", self.tr("HOUSE_LINE_COLOR"), SettingsOption.COLOR),
			("HOUSE_LINE_WIDTH", self.tr("HOUSE_LINE_WIDTH"), SettingsOption.SIZE),
			("ASCENDANT_LINE_COLOR", self.tr("ASCENDANT_LINE_COLOR"), SettingsOption.COLOR),
			("ASCENDANT_LINE_WIDTH", self.tr("ASCENDANT_LINE_WIDTH"), SettingsOption.SIZE),
			("MOEDIUM_COELI_LINE_COLOR", self.tr("MOEDIUM_COELI_LINE_COLOR"), SettingsOption.COLOR),
			("MOEDIUM_COELI_LINE_WIDTH", self.tr("MOEDIUM_COELI_LINE_WIDTH"), SettingsOption.SIZE),
			("INNER_POINTERS_LENGTH", self.tr("INNER_POINTERS_LENGTH"), SettingsOption.SIZE),
			("OUTER_POINTERS_LENGTH", self.tr("OUTER_POINTERS_LENGTH"), SettingsOption.SIZE),
			("ANGLE_POINTERS_WIDTH", self.tr("ANGLE_POINTERS_WIDTH"), SettingsOption.SIZE),
			("ZODIAC_SIGNS_OFFSET", self.tr("ZODIAC_SIGNS_OFFSET"), SettingsOption.SIZE),
			("PLANETS_OFFSET", self.tr("PLANETS_OFFSET"), SettingsOption.SIZE),
			("TRANSIT_PLANETS_OFFSET", self.tr("TRANSIT_PLANETS_OFFSET"), SettingsOption.SIZE),
			("CONJUNCTION_WIDTH", self.tr("CONJUNCTION_WIDTH"), SettingsOption.SIZE),
			("OPPOSITION_WIDTH", self.tr("OPPOSITION_WIDTH"), SettingsOption.SIZE),
			("TRINE_WIDTH", self.tr("TRINE_WIDTH"), SettingsOption.SIZE),
			("SQUARE_WIDTH", self.tr("SQUARE_WIDTH"), SettingsOption.SIZE),
			("SEXTILE_WIDTH", self.tr("SEXTILE_WIDTH"), SettingsOption.SIZE),
			("SEMISEXTILE_WIDTH", self.tr("SEMISEXTILE_WIDTH"), SettingsOption.SIZE)
		)


class SettingsForm(QTabWidget):

	optionsChanged = pyqtSignal()

	def __init__(self):
		super().__init__(parent = None)
		self.setWindowTitle(self.tr("Settings"))
		self.layoutOptionsScroll = 0
		self.symbolsColorsScroll = 0
		self.zodiacOptionsScroll = 0
		self.fontOptionsScroll = 0
		self.setTabs()

	def setTabs(self):
		self.optionsList = OptionsList()
		self.layoutOptions = SettingsFormTab(self.optionsList.LAYOUT_OPTIONS)
		self.layoutOptions.verticalScrollBar().setValue(self.layoutOptionsScroll)
		self.symbolsColors = SettingsFormTab(self.optionsList.SYMBOLS_COLORS)
		self.symbolsColors.verticalScrollBar().setValue(self.symbolsColorsScroll)
		self.zodiacOptions = SettingsFormTab(self.optionsList.ZODIAC_OPTIONS)
		self.zodiacOptions.verticalScrollBar().setValue(self.zodiacOptionsScroll)
		self.fontOptions = SettingsFormTab(self.optionsList.FONT_OPTIONS)
		self.fontOptions.verticalScrollBar().setValue(self.fontOptionsScroll)
		self.addTab(self.layoutOptions, self.tr("layout_options"))
		self.addTab(self.symbolsColors, self.tr("symbols_colors"))
		self.addTab(self.zodiacOptions, self.tr("zodiac_options"))
		self.addTab(self.fontOptions, self.tr("font_options"))

		for i in range(self.count()):
			self.widget(i).optionsChanged[str, str].connect(self.optionsChanged.emit)
			self.widget(i).optionsChanged[str, float].connect(self.optionsChanged.emit)

	def reset(self):
		self.layoutOptionsScroll = self.layoutOptions.verticalScrollBar().value()
		self.symbolsColorsScroll = self.symbolsColors.verticalScrollBar().value()
		self.zodiacOptionsScroll = self.zodiacOptions.verticalScrollBar().value()
		self.fontOptionsScroll = self.fontOptions.verticalScrollBar().value()

		while self.count() >= 1:
			first_widget = self.widget(0)
			self.removeTab(0)
			del first_widget#in this way we actually destroy the widgets instead of merely removing them
		self.setTabs()
			

class SettingsWindow(QMainWindow):

	optionsChanged = pyqtSignal()		

	def __init__(self):
		super().__init__()
		self.setWindowTitle(self.tr("settings_window"))
		self.settingsClass = cruscoplanets.cmdline.svgSettings()
		self.settingsForm = SettingsForm()
		self.settingsForm.optionsChanged.connect(self.optionsChanged.emit)
		self.setCentralWidget(self.settingsForm)
		
		self.toolbar = self.addToolBar(self.tr("actions"))
		save_icon = self.style().standardIcon(QStyle.SP_DialogSaveButton)
		self.save = self.toolbar.addAction(self.tr("save"))
		self.save.setIcon(save_icon)
		self.save.triggered.connect(self.settingsClass.save)
		reset_icon = self.style().standardIcon(QStyle.SP_DialogResetButton)
		self.reset = self.toolbar.addAction(self.tr("reset"))
		self.reset.setIcon(reset_icon)
		self.reset.triggered.connect(self._onReset)
		
	def _onReset(self):
		self.settingsClass.reset()
		self.settingsForm.reset()
		self.optionsChanged.emit()



#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtc

		
class FilterError(Exception):

	def __init__(self, message):
		super().__init__()
		
	def __str__(self):
		return self.message
		

class Filter:

	STARTS_BY = 0
	ENDS_BY = 1
	EQUALS = 2
	DIFFERENT_FROM = 3
	LESS_THAN = 4
	GREATER_THAN = 5
	LESS_OR_EQUAL = 6
	GREATER_OR_EQUAL = 7
	
	def __init__(self, column, relation: int, control_value):
		self.column = column#this can be a column index or name
		self.relation = relation
		self.controlValue = control_value
		
	def __repr__(self):
		ret_str = "%s: value "%self.column
		if self.relation == self.__class__.STARTS_BY:
			ret_str += 'starts by '
		elif self.relation == self.__class__.ENDS_BY:
			ret_str += 'ends by '
		elif self.relation == self.__class__.EQUALS:
			ret_str += '== '
		elif self.relation == self.__class__.DIFFERENT_FROM:
			ret_str += '!= '
		elif self.relation == self.__class__.LESS_THAN:
			ret_str += '< '
		elif self.relation == self.__class__.GREATER_THAN:
			ret_str += '> '
		elif self.relation == self.__class__.LESS_OR_EQUAL:
			ret_str += '<= '
		elif self.relation == self.__class__.GREATER_OR_EQUAL:
			ret_str += '>= '
		ret_str += "%r"%self.controlValue
		return ret_str
		
	def check(self, value, case_sensitive = False) -> bool:
		"""Takes a value as argument and controls if it respects the filter, returning True if it is an acceptable value, False if it isn't."""

		control_value = self.controlValue
		if not case_sensitive:
			if type(value) == str:
				value = value.lower()
			if type(control_value) == str:
				control_value = control_value.lower()
		if self.relation in {self.__class__.STARTS_BY, self.__class__.ENDS_BY}:
			if type(value) != str:
				raise FilterError("STARTS_BY and ENDS_BY relations can be applied only to strings")
			else:
				control_value_length = len(control_value)
				if self.relation == self.__class__.STARTS_BY:
					return control_value == value[0:control_value_length]
				else:
					if control_value == '':
						return True
					else:
						return control_value == value[-control_value_length:]
		else:
			if self.relation == self.__class__.EQUALS:
				return control_value == value
			elif self.relation == self.__class__.DIFFERENT_FROM:
				return control_value != value
			elif self.relation == self.__class__.LESS_THAN:
				return value < control_value
			elif self.relation == self.__class__.GREATER_THAN:
				return value > control_value
			elif self.relation == self.__class__.LESS_OR_EQUAL:
				return value <= control_value
			elif self.relation == self.__class__.GREATER_OR_EQUAL:
				return value >= control_value


class FilterWidget(qtw.QWidget):

	def __init__(self):
		super().__init__()


class DtStringFilterWidget(FilterWidget):

	filterChanged = qtc.pyqtSignal(int, int, int, int, int)#last int to say if the filter is enabled (1) or disabled (0)

	def __init__(self):
		super().__init__()
		self.applyStart = qtw.QCheckBox(self.tr("apply"))
		self.applyStart.setChecked(False)
		self.applyEnd = qtw.QCheckBox(self.tr("apply"))
		self.applyEnd.setChecked(False)
		self.startDate = qtw.QDateEdit(qtc.QDate(1900, 1, 1))
		self.startDate.setCalendarPopup(True)
		self.startDate.setDisplayFormat("dd/MM/yyyy")
		self.startDate.setDisabled(True)
		self.endDate = qtw.QDateEdit(qtc.QDate.currentDate())
		self.endDate.setCalendarPopup(True)
		self.endDate.setDisplayFormat("dd/MM/yyyy")
		self.endDate.setDisabled(True)
		layout = qtw.QGridLayout()
		layout.setContentsMargins(0, 0, 0, 0)
		layout.addWidget(qtw.QLabel(self.tr("start_date")), 0, 0)
		layout.addWidget(self.startDate, 0, 1)
		layout.addWidget(self.applyStart, 0, 2)
		layout.addWidget(qtw.QLabel(self.tr("end_date")), 1, 0)
		layout.addWidget(self.endDate, 1, 1)
		layout.addWidget(self.applyEnd, 1, 2)
		self.setLayout(layout)
		self.applyStart.clicked.connect(self._onApplyStartClicked)
		self.applyEnd.clicked.connect(self._onApplyEndClicked)
		self.applyStart.clicked.connect(self._onDatesChanged)
		self.applyEnd.clicked.connect(self._onDatesChanged)
		self.startDate.dateChanged.connect(self._onDatesChanged)
		self.endDate.dateChanged.connect(self._onDatesChanged)
		
	def _onDatesChanged(self):
		start_enabled = 1 if self.startDate.isEnabled() else 0
		start_date = self.startDate.date()
		self.filterChanged.emit(start_date.year(), start_date.month(), start_date.day(), Filter.GREATER_THAN, start_enabled)
		end_enabled = 1 if self.endDate.isEnabled() else 0
		end_date = self.endDate.date()
		self.filterChanged.emit(end_date.year(), end_date.month(), end_date.day(), Filter.LESS_THAN, end_enabled)
	
	def _onApplyStartClicked(self):
		if self.applyStart.isChecked():
			self.startDate.setEnabled(True)
		else:
			self.startDate.setEnabled(False)
	
	def _onApplyEndClicked(self):
		if self.applyEnd.isChecked():
			self.endDate.setEnabled(True)
		else:
			self.endDate.setEnabled(False)


class SexFilterWidget(FilterWidget):

	filterChanged = qtc.pyqtSignal(int, int)

	def __init__(self):
		super().__init__()
		self.relationComboBox = qtw.QComboBox()
		self.sexComboBox = qtw.QComboBox()
		self.relationComboBox.addItem(self.tr("equals_to"))
		self.relationComboBox.addItem(self.tr("different_from"))
		self.relationComboBox.addItem(self.tr("don't_apply"))
		self.relationComboBox.setCurrentIndex(2)
		self.sexComboBox.addItem(self.tr("male"))
		self.sexComboBox.addItem(self.tr("female"))
		self.sexComboBox.addItem(self.tr("undefined"))
		layout = qtw.QVBoxLayout()
		layout.setContentsMargins(0, 0, 0, 0)
		layout.addWidget(self.relationComboBox)
		layout.addWidget(self.sexComboBox)
		self.setLayout(layout)
		self.sexComboBox.currentIndexChanged.connect(self._onChange)
		self.relationComboBox.currentIndexChanged.connect(self._onChange)
	
	def _onChange(self, thevalue):
		if self.relationComboBox.currentIndex() == 0:
			relation = Filter.EQUALS
		elif self.relationComboBox.currentIndex() == 1:
			relation = Filter.DIFFERENT_FROM
		else:
			relation = -1
		sex = self.sexComboBox.currentIndex() if self.sexComboBox.currentIndex() != 2 else -1
		self.filterChanged.emit(relation, sex)
	

class TextFilterWidget(FilterWidget):

	filterChanged = qtc.pyqtSignal(int, str)
		
	def __init__(self):
		super().__init__()
		self.lineEdit = qtw.QLineEdit()
		self.lineEdit.textChanged.connect(self._onTextChanged)
		self.comboBox = qtw.QComboBox()
		self.comboBox.addItem(self.tr("starts_by"))
		self.comboBox.addItem(self.tr("ends_by"))
		self.comboBox.addItem(self.tr("equals_to"))
		self.comboBox.addItem(self.tr("different_from"))
		self.comboBox.addItem(self.tr("don't_apply"))
		self.comboBox.currentIndexChanged.connect(self._onConditionChanged)
		layout = qtw.QVBoxLayout()
		layout.setContentsMargins(0, 0, 0, 0)
		layout.addWidget(self.comboBox)
		layout.addWidget(self.lineEdit)
		self.setLayout(layout)

	def emitChange(self):
		if self.comboBox.currentIndex() == 0:
			relation = Filter.STARTS_BY
		elif self.comboBox.currentIndex() == 1:
			relation = Filter.ENDS_BY
		elif self.comboBox.currentIndex() == 2:
			relation = Filter.EQUALS
		elif self.comboBox.currentIndex() == 3:
			relation = Filter.DIFFERENT_FROM
		else:
			relation = -1
		value = self.lineEdit.text()
		self.filterChanged.emit(relation, value)
		
	def _onTextChanged(self, value):
		self.emitChange()

	def _onConditionChanged(self, value):
		self.emitChange()




#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout, QSizePolicy, QMessageBox, QStyle
from PyQt5 import QtSvg
from PyQt5.QtSql import QSqlDatabase
from . import chart_form
from . import svg_widget
import cruscoplanets.cmdline
from cruscoplanets.chart_printer import FormatError
from .. import config


class Chart(QWidget):
	
	def __init__(self, name: str = None, surname: str = None, sex: int = None, birthtime: str = None, birthlocation_id: int = None):
		super().__init__()
		self.configuration = config.Configuration()

	def export(self, fileName):
		self.svgWidget.export(fileName)
		
	def _handleExportError(self, fileFormat):
		message = QMessageBox(QMessageBox.Critical, self.tr("error"), self.tr(f"{fileFormat}_is_not_an_allowed_format".format(fileFormat)))
		message.exec_()


class BirthChart(Chart):

	def __init__(self, name: str = None, surname: str = None, sex: int = None, birthtime: str = None, birthlocation_id: int = None):
		super().__init__(name, surname, sex, birthtime, birthlocation_id)
		self.chartForm = chart_form.BirthChartForm(name, surname, sex, birthtime, birthlocation_id)
		self.chartForm.submitData.connect(lambda data: self._onDataSubmitted(data))
		
		fillWidget = QWidget()
		fillWidget.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
		
		formLayout = QVBoxLayout()
		formLayout.addWidget(self.chartForm)
		formLayout.addWidget(fillWidget)
		
		self.svgWidget = svg_widget.SvgWidget()
		self._onDataSubmitted(self.chartForm.voidchartdata())
		self.layout = QHBoxLayout()
		self.layout.addLayout(formLayout)
		self.layout.addWidget(self.svgWidget)
		self.setLayout(self.layout)

	def _onDataSubmitted(self, data: tuple):
		svgBuffer = cruscoplanets.cmdline.doBirthBuffer(*data)
		self.svgWidget.load(svgBuffer)
		
	def export(self, fileName):
		birthtime, birthlocation, name, surname, sex, language = self.chartForm.chartdata()
		try:
			chartprinter = cruscoplanets.cmdline.doBirthChart(birthtime, birthlocation, name, surname, sex, fileName, self.configuration.CHART_DEFAULT_LANGUAGE)
			message = QMessageBox(QMessageBox.NoIcon, self.tr("success"), self.tr("export_successful"))
			message.exec_()
		except FormatError as fe:
			self._handleExportError(fe.format)


class TransitChart(Chart):

	def __init__(self, name: str = None, surname: str = None, sex: int = None, birthtime: str = None, birthlocation_id: int = None):
		super().__init__(name, surname, sex, birthtime, birthlocation_id)
		self.chartForm = chart_form.TransitChartForm(name, surname, sex, birthtime, birthlocation_id)
		self.chartForm.submitData.connect(lambda data: self._onDataSubmitted(data))
		
		fillWidget = QWidget()
		fillWidget.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
		
		formLayout = QVBoxLayout()
		formLayout.addWidget(self.chartForm)
		formLayout.addWidget(fillWidget)
		
		self.svgWidget = svg_widget.SvgWidget()
		self._onDataSubmitted(self.chartForm.voidchartdata())
		self.layout = QHBoxLayout()
		self.layout.addLayout(formLayout)
		self.layout.addWidget(self.svgWidget)
		self.setLayout(self.layout)

	def _onDataSubmitted(self, data: tuple):
		svgBuffer = cruscoplanets.cmdline.doTransitBuffer(*data)
		self.svgWidget.load(svgBuffer)

	def export(self, fileName):
		birthtime, birthlocation, transittime, transitlocation, name, surname, sex, language = self.chartForm.chartdata()
		try:
			chartprinter = cruscoplanets.cmdline.doTransitChart(birthtime, birthlocation, transittime, transitlocation, name, surname, sex, fileName, self.configuration.CHART_DEFAULT_LANGUAGE)
			message = QMessageBox(QMessageBox.NoIcon, self.tr("success"), self.tr("export_successful"))
			message.exec_()
		except FormatError as fe:
			self._handleExportError(fe.format)



#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
from PyQt5.QtCore import Qt, QObject, pyqtSignal, QDate, QTime, QDateTime
from PyQt5.QtSql import QSqlDatabase, QSqlQuery
import cruscoplanets.cmdline
from .. import config
from . import database_filters


class IdFieldError(Exception):

	def __init__(self):
		super().__init__()
		
	def __str__(self):
		return "ID value can not be edited"


class FieldError(Exception):

	def __init__(self):
		super().__init__()
		
	def __str__(self):
		return "Fields can be set only with the setValue() function"


class Field:

	def __init__(self, name, value):
		self._name = name
		self._value = value
		
	def name(self):
		return self._name
		
	def value(self):
		return self._value
		
	def setValue(self, new_value):
		self._value = new_value
		
	def __repr__(self):
		return "(%s, %s)"%(self._name, self._value)
		
	def __lt__(self, other):
		return self._value < other._value
		
	def __eq__(self, other):
		return self._value == other._value
		
	def valueForFilters(self):
		return self.value()
		

class IdField(Field):

	def __init__(self, name, value):
		super().__init__(name, value)
		
	def setValue(self, new_value):
		return IdFieldError()
		

class SexField(Field):

	def __init__(self, name, value):
		super().__init__(name, value)
		

class LocationField(Field):

	def __init__(self, name, value):
		super().__init__(name, value)
		

class DtStringField(Field):

	def __init__(self, name, value):
		super().__init__(name, value)
		self.is_time_local, self.is_time_known, year, month, day, hour, minute, second = cruscoplanets.cmdline.parseDatetimeString(value)
		self._qdatetime = QDateTime(QDate(year, month, day,), QTime(hour, minute, second))
		
	def valueForFilters(self):
		return self._qdatetime
		
	def __lt__(self, other):
		return self._qdatetime < other._qdatetime
		
	def __eq__(self, other):
		return self._qdatetime == other._qdatetime


class FieldsDict:

	def __init__(self, fields, values):
		self.fieldNames = fields.names()
		_fields = []
		for i in range(len(fields)):
			fieldClass = fields[i].fieldClass()
			_fields.append(fieldClass(fields[i].name(), values[i]))
		self._fields = tuple(_fields)
		
	def __getitem__(self, index):
		if type(index) == int:
			return self._fields[index]
		elif type(index) == str:
			the_index = self.fieldNames.index(index)
			return self._fields[the_index]

	def __setitem__(self, index, value):
		return FieldError("Can not set a new value to a field")
		
	def __iter__(self):
		return iter(self._fields)
		
	def __len__(self):
		return len(self._fields)


class Record(QObject):

	def __init__(self, values, columns):
		self._columns = columns
		self._values = values
		self.fields = FieldsDict(self._columns, values)
		
	def __repr__(self):
		retstr = '(' + ', '.join([field.__repr__() for field in self.fields]) + ')'
		return retstr
		
	def valuesString(self):
		retstr = '(' + ', '.join(["%r"%field.value() for field in self.fields]) + ')'
		return retstr
		
	def _applyFilter(self, thefilter: database_filters.Filter):
		value = self.fields[thefilter.column].valueForFilters()
		return thefilter.check(value)
	
	def primaryKey(self):
		return self.fields[self._columns.primaryKey().index()].value()
		
		
class Column:

	def __init__(self, index, name, type_val, not_null, default, primary_key):
		self._index = index
		self._name = name
		self._header = name
		self._type = type_val
		self._isNotNull = not_null == 1
		self._defaultValue = default
		self._isPrimaryKey = primary_key == 1
		self._fieldClass = IdField if self._isPrimaryKey else Field
		
	def fieldClass(self):
		return self._fieldClass
		
	def setFieldClass(self, the_class):
		self._fieldClass = the_class
	
	def index(self):
		return self._index
	
	def name(self):
		return self._name
	
	def type(self):
		return self._type
	
	def isNotNull(self):
		return self._isNotNull
	
	def defaultValue(self):
		return self._defaultValue
	
	def isPrimaryKey(self):
		return self._isPrimaryKey
		
	def header(self):
		return self._header
	
	def setHeader(self, value):
		self._header = value
		
	def __repr__(self):
		return "%s, %s, not null: %r, default: %r, primary key: %r"%(self.name(), self.type(), self.isNotNull(), self.defaultValue(), self.isPrimaryKey())
		
		
class Columns:

	def __init__(self, col_list):
		self._columns = tuple(col_list)
		
	def indexes(self):
		return tuple(column.index() for column in self._columns)
		
	def names(self):
		return tuple(column.name() for column in self._columns)
		
	def types(self):
		return tuple(column.type() for column in self._columns)
		
	def defaultValues(self):
		return tuple(column.defaultValue() for column in self._columns)
		
	def primaryKey(self):
		ret = None
		for column in self._columns:
			if column.isPrimaryKey() == True:
				ret = column
		return ret
		
	def __repr__(self):
		return '(' + ', '.join(self.names()) + ')'
		
	def __getitem__(self, index_or_name):
		if type(index_or_name) == str:
			index = self.names().index(index_or_name)
		elif type(index_or_name) == int:
			index = index_or_name
		return self._columns[index]
		
	def __iter__(self):
		return iter(self._columns)
		
	def __len__(self):
		return len(self._columns)


class TableModelError(Exception):

	def __init__(self, message):
		super().__init__()
		self.message = message
	
	def __str__(self):
		return self.message
		
		
class TableModel(QObject):

	deleted = pyqtSignal(int)
	updated = pyqtSignal(dict)
	added = pyqtSignal(dict)

	def __init__(self, db, table_name):
		super().__init__()
		self.db = db
		self.tableName = table_name
		self.setColumns()
		self.refresh()
		self.filters = []

	def setColumns(self):
		columns_query = "PRAGMA table_info(%s)"%self.tableName
		columns_result = self.db.exec(columns_query)
		columns = []
		while columns_result.next():
			columns.append(Column(columns_result.value(0), columns_result.value(1), columns_result.value(2), columns_result.value(3), columns_result.value(4), columns_result.value(5)))
		self.columns = Columns(columns)

	def addFilter(self, thefilter: database_filters.Filter):
		self.filters.append(thefilter)
		
	def _applyFilter(self, thefilter: database_filters.Filter, data):
		return filter(lambda record: record._applyFilter(thefilter), data)
		
	def select(self):
		data = self.records
		for thefilter in self.filters:
			data = self._applyFilter(thefilter, data)
		return tuple(data)
		
	def sortByField(self, index_or_name, reverse: bool = False):
		self.records = sorted(self.records, key = lambda record: record.fields[index_or_name], reverse = reverse)
		
	def refresh(self):
		query = ("SELECT * FROM %s"%self.tableName)
		result = self.db.exec(query)
		values_number = result.record().count()
		records = []
		while result.next():
			record = []
			for i in range(values_number):
				record.append(result.value(i))
			records.append(record)
		self.records = tuple(Record(record, self.columns) for record in records)
		#now we cast the primary key field in each record to the IdField class
		
	def updateRecord(self, **kwargs):
		primary_key = self.columns.primaryKey().name()
		if primary_key not in kwargs.keys():
			raise TableModelError("A value for '%s' must be set where updating"%primary_key)
			
		set_values = {key: value for key, value in kwargs.items() if key != primary_key}
		if len(set_values.keys()) == 0:
			print("No field to update has been specified")
			return

		set_values = self._sortByColumnsOrder(set_values.items())
		keys = tuple(value[0] for value in set_values)
		bindParams = tuple(":%s"%key for key in keys)
		expressions = tuple("%s = %s"%(keys[i], bindParams[i]) for i in range(len(keys)))
		values = tuple(value[1] for value in set_values)
		query_str = "UPDATE %s SET "%self.tableName
		query_str += ', '.join(expressions)
		query_str += " WHERE "
		query_str += "%s = :%s"%(primary_key, primary_key)
		query = QSqlQuery(self.db)
		query.prepare(query_str)
		for i in range(len(values)):
			query.bindValue(bindParams[i], values[i])
		query.bindValue(":%s"%primary_key, kwargs[primary_key])
		success = query.exec()
		if success:
			self.updated.emit(kwargs)
			self.refresh()
		else:
			print(query.lastError().text())
		return success

	def deleteRecord(self, id_value):
		query_str = "DELETE FROM " + self.tableName + " WHERE " + self.columns.primaryKey().name() + " = :" + self.columns.primaryKey().name()
		query = QSqlQuery(self.db)
		query.prepare(query_str)
		query.bindValue(":" + self.columns.primaryKey().name(), id_value)
		success = query.exec()
		if success:
			self.deleted.emit(id_value)
			self.refresh()
		else:
			print(query.lastError().text())
		return success
		
	def _deleteByFieldAndValue(self, field, value):
		select_str = "SELECT " + self.columns.primaryKey().name() + " FROM " + self.tableName + " WHERE " + field + " = :" + field
		select = QSqlQuery(self.db)
		select.prepare(select_str)
		select.bindValue(":%s"%field, value)
		select.exec()
		id_values = []
		while select.next():
			id_values.append(select.value(0))
		success = True
		for value in id_values:
			success = self.deleteRecord(value)
		return success
			
	def _sortByColumnsOrder(self, pairs):
		"""Sorts a sequence of key-value pairs, where the keys are the column names, based on the order of the columns in the table"""
		return sorted(pairs, key = lambda couple: self.columns.names().index(couple[0]))
		
	def addRecord(self, **kwargs):
		pairs = self._sortByColumnsOrder(kwargs.items())
		keys = tuple(pair[0] for pair in pairs)
		bindParams = tuple(":%s"%key for key in keys)
		values = tuple(pair[1] for pair in pairs)
		query_str = "INSERT INTO " + self.tableName 
		query_str += "(" + ', '.join(keys) + ')'
		query_str += " VALUES (" + ', '.join(bindParams) + ')'
		query = QSqlQuery(self.db)
		query.prepare(query_str)
		for i in range(len(values)):
			query.bindValue(bindParams[i], values[i])
		success = query.exec()
		if success:
			self.added.emit(kwargs)
			self.refresh()
		else:
			print(query.lastError().text())
		return success
			
	def __repr__(self):
		ret_str = self.columns.__repr__()
		ret_str += '\n' +'-'*len(ret_str) + '\n'
		ret_str += '\n'.join([record.valuesString() for record in self.select()])
		return ret_str			


class PeopleTableModel(TableModel):

	def __init__(self, db):
		super().__init__(db, 'people')

	def setColumns(self):
		super().setColumns()
		self.columns['name'].setHeader(self.tr('name'))
		self.columns['surname'].setHeader(self.tr('surname'))
		self.columns['sex'].setHeader(self.tr('sex'))
		self.columns['sex'].setFieldClass(SexField)
		self.columns['birthdtstring'].setHeader(self.tr('birthdtstring'))
		self.columns['birthdtstring'].setFieldClass(DtStringField)
		self.columns['birthlocationid'].setHeader(self.tr('birthlocationid'))
		self.columns['birthlocationid'].setFieldClass(LocationField)
		
	def addRecord(self, name, surname, sex, birthdtstring, birthlocationid):
		kwargs = {
			'id': None,#id None will set automatically the value as autoincrement in the SQL database
			'name': name,
			'surname': surname,
			'sex': self.columns['sex'].defaultValue() if sex == None else sex,
			'birthdtstring': birthdtstring if birthdtstring != None else config.Configuration().DEFAULT_DTSTRING,
			'birthlocationid': birthlocationid if birthlocationid != None else config.Configuration().DEFAULT_LOCATION
		}
		return super().addRecord(**kwargs)
	
	def addVoidRecord(self):
		return self.addRecord(name = None, surname = None, sex = None, birthdtstring = None, birthlocationid = None)
		

class PersonalDatabaseModel(QObject):

	def __init__(self):
		super().__init__()
		self.personaldb = QSqlDatabase.addDatabase("QSQLITE")
		self.personaldb.setDatabaseName(config.Configuration().databaseFilePath())
		self._initDatabase()
		self.peopleTable = PeopleTableModel(self.personaldb)

	def _initDatabase(self):
		if not self.personaldb.open():
			print(self.personaldb.lastError().text())
			sys.exit(1)
		self.personaldb.exec("""
		CREATE TABLE IF NOT EXISTS people (
			id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
			name TEXT,
			surname TEXT,
			sex INTEGER CHECK( sex IN (-1, 0, 1)) NOT NULL DEFAULT -1,
			birthdtstring TEXT NOT NULL,
			birthlocationid INT NOT NULL
		);
		""")



#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
import cruscoplanets.utils as cruscoutils
from PyQt5.QtCore import QDate, QTime
import os
from . import resources


class Configuration(metaclass=cruscoutils.Singleton):

	CONFIG_FILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), "config", "base.txt")
	DB_DIR = os.path.join(os.path.dirname(__file__), "databases")
	PERSONAL_DB_FILENAME = "personal.db"

	def __init__(self):
		self.reset()
		for directory in (self.__class__.DB_DIR,):
			os.makedirs(directory, exist_ok = True)#argument exist_ok set to True makes the function not alter the directories if existent
	
	def reset(self):
		config = cruscoutils.parseBanalConfig(self.__class__.CONFIG_FILE)
		for line in config:
			if line[1].isnumeric():
				self.__dict__[line[0]] = float(line[1])
			else:
				self.__dict__[line[0]] = line[1]
	
	def aboutFilePath(self):
		return ':/about/about/%s.md'%self.GUI_DEFAULT_LANGUAGE
		
	def databaseFilePath(self):
		return os.path.join(self.__class__.DB_DIR, self.__class__.PERSONAL_DB_FILENAME)

	def minimumDate(self):
		year, month, day = tuple(int(n) for n in self.MINIMUM_DATE.split('-'))
		return QDate(year, month, day)
	
	def defaultTime(self):#should most conveniently be set to 12:00
		return QTime(12, 0, 0)
		
	def guiLanguages(self):
		return tuple(self.GUI_LANGUAGES.split())
		
	def chartLanguages(self):
		return tuple(self.CHART_LANGUAGES.split())
		
	def __repr__(self):
		return '\n'.join(["%s: %s"%(key, value) for key, value in self.__dict__.items()])
		
	def save(self):
		new_config = cruscoutils.buildBanalConfig(self.__dict__.items())
		with open(self.__class__.CONFIG_FILE, 'w') as config_file:
			config_file.write(new_config)
	
	def saveDefaultLanguage(self, language):
		self.GUI_DEFAULT_LANGUAGE = language
		new_config = dict(cruscoutils.parseBanalConfig(self.__class__.CONFIG_FILE))
		new_config["GUI_DEFAULT_LANGUAGE"] = language
		new_config = cruscoutils.buildBanalConfig(new_config.items())
		with open(self.__class__.CONFIG_FILE, 'w') as config_file:
			config_file.write(new_config)



#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
from PyQt5.QtWidgets import QMainWindow, QWidget, QComboBox, QFormLayout, QStyle, QMessageBox, QLabel
from PyQt5.QtCore import pyqtSignal, QObject, Qt
from .. import config
import langcodes


class LanguageComboBox(QComboBox):

	languageChanged = pyqtSignal(str, str)
	
	def __init__(self, languages, defaultLanguage):
		super().__init__()
		self.configuration = config.Configuration()
		self.defaultLanguage = (defaultLanguage, langcodes.Language.get(defaultLanguage).display_name(self.configuration.GUI_DEFAULT_LANGUAGE))
		self.codesAndNames = []
		for language in languages:
			lang = langcodes.Language.get(language)
			self.codesAndNames.append((language, lang.display_name(self.configuration.GUI_DEFAULT_LANGUAGE)))
		
		for language in self.codesAndNames:
			self.addItem(language[1])

		self.reset()
		self.currentIndexChanged.connect(lambda index: self.languageChanged.emit(*self.codesAndNames[index]))
	
	def reset(self):
		self.setCurrentIndex(self.codesAndNames.index(self.defaultLanguage))
			

class LanguageSettingsWindow(QMainWindow):

	chartLanguageUpdated = pyqtSignal()		

	def __init__(self):
		super().__init__()
		self.setWindowTitle(self.tr("language_settings_window"))
		self.resize(400, 400)
		self.configuration = config.Configuration()
		self.container = QWidget(self)
		layout = QFormLayout()
		self.chartComboBox = LanguageComboBox(self.configuration.chartLanguages(), self.configuration.CHART_DEFAULT_LANGUAGE)
		self.guiComboBox = LanguageComboBox(self.configuration.guiLanguages(), self.configuration.GUI_DEFAULT_LANGUAGE)
		self.chartComboBox.languageChanged.connect(self._updateChartLanguage)
		self.guiComboBox.languageChanged.connect(self._updateGuiLanguage)
		self.notifications = QLabel()
		layout.addRow(self.tr("chart_language"), self.chartComboBox)
		layout.addRow(self.tr("gui_language"), self.guiComboBox)
		layout.addRow(self.notifications)
		self.container.setLayout(layout)
		self.setCentralWidget(self.container)

		self.toolbar = self.addToolBar(self.tr("actions"))
		save_icon = self.style().standardIcon(QStyle.SP_DialogSaveButton)
		self.save = self.toolbar.addAction(self.tr("save"))
		self.save.setIcon(save_icon)
		self.save.triggered.connect(self.configuration.save)
		reset_icon = self.style().standardIcon(QStyle.SP_DialogResetButton)
		self.reset = self.toolbar.addAction(self.tr("reset"))
		self.reset.setIcon(reset_icon)
		self.reset.triggered.connect(self._onReset)


	def _updateChartLanguage(self, langcode, langname):
		self.configuration.CHART_DEFAULT_LANGUAGE = langcode
		self.chartLanguageUpdated.emit()

	def _updateGuiLanguage(self, langcode, langname):
		self.configuration.saveDefaultLanguage(langcode)
		self.notifications.setText(self.tr("language_will_be_changed_to_{0}_at_next_loading").format(langname))

	def _onReset(self):
		self.configuration.reset()
		self.chartComboBox.reset()
		self.guiComboBox.reset()
		self.chartLanguageUpdated.emit()



#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QFormLayout, QLineEdit, QRadioButton


class PersonalDataForm(QWidget):

	def __init__(self, name: str = None, surname: str = None, sex: int = None):
		super().__init__()

		self.baseLayout = QFormLayout()
		
		self.nameField = QLineEdit(self)
		if name != None:
			self.nameField.setText(name)
		self.surnameField = QLineEdit(self)
		if surname != None:
			self.surnameField.setText(surname)
		sexWidget = QWidget(self)#parent for sex radio buttons
		self.maleRadio = QRadioButton(self.tr("male"), sexWidget)
		self.femaleRadio = QRadioButton(self.tr("female"), sexWidget)
		self.undefinedRadio = QRadioButton(self.tr("undefined"), sexWidget)
		if sex == 0:
			self.maleRadio.setChecked(True)
		if sex == 1:
			self.femaleRadio.setChecked(True)
		else:
			self.undefinedRadio.setChecked(True)
		
		sexLayout = QHBoxLayout()
		sexLayout.addWidget(self.maleRadio)
		sexLayout.addWidget(self.femaleRadio)
		sexLayout.addWidget(self.undefinedRadio)
		sexWidget.setLayout(sexLayout)
		
		self.baseLayout = QFormLayout()
		self.baseLayout.addRow(self.tr("name"), self.nameField)
		self.baseLayout.addRow(self.tr("surname"), self.surnameField)
		self.baseLayout.addRow(self.tr("sex"), sexWidget)
		self.setLayout(self.baseLayout)
	
	@property
	def sex(self):
		if self.maleRadio.isChecked():
			return 0
		elif self.femaleRadio.isChecked():
			return 1
		else:
			return -1
			
	def setSex(self, sex_int):
		if sex_int == 0:
			self.maleRadio.setChecked(True)
		elif sex_int == 1:
			self.femaleRadio.setChecked(True)
		else:
			self.undefinedRadio.setChecked(True)



import os
import setuptools

if __name__ == '__main__':
	with open("README.md", 'r') as long_file:
		long_descript = long_file.read()
	setuptools.setup(
		name='cruscoplanets_gui',
		version='0.0.3',
		packages=setuptools.find_packages(),
		entry_points={
			'gui_scripts': [
				'cruscoplanets_gui = cruscoplanets_gui.__main__:main',
			],
		},
		install_requires=[
			'wheel',
			'cruscoplanets>=0.3.0',
			'langcodes[data]',
			'pycountry==22.3.5',
			'PyQt5==5.15.4'
		],
		python_requires='==3.8.10',
		package_data={'cruscoplanets_gui': ['config/base.txt']},
		author='Emiliano Minerba',
		author_email='emi.nerba@gmail.com',
		description="GUI interface for the cruscoplanets package",
		long_description=long_descript,
		long_description_content_type='text/markdown',
		license='GPL',
		url='https://gitlab.com/kikulacho92/cruscoplanets_gui',
		keywords='astrology Morpurgo dialectic charts'
	)



#    This file is part of Cruscoplanets GUI.
#
#   Cruscoplanets GUI is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Cruscoplanets GUI is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Cruscoplanets GUI.  If not, see <http://www.gnu.org/licenses/>.
from PyQt5.QtWidgets import QLineEdit, QCompleter, QWidget, QHBoxLayout, QVBoxLayout, QGridLayout, QPushButton, QLabel, QMainWindow
from PyQt5.QtCore import QStringListModel, pyqtSignal
import cruscoplanets.cmdline
import cruscoplanets.angles
import pycountry

class LocationEntry:

	def __init__(self, locfromdatabase):
		self.id = locfromdatabase['id']
		self.name = locfromdatabase['name']
		self.country = locfromdatabase['country']
		self.countryName = pycountry.countries.get(alpha_2=self.country).name
		self.timezone = locfromdatabase['timezone']
		self.latitude = cruscoplanets.angles.Angle(locfromdatabase['latitude'])
		self.longitude = cruscoplanets.angles.Angle(locfromdatabase['longitude'])
	
	@property	
	def labelString(self):
		return "%s - %s (%s - %s, %s)"%(self.name, self.countryName, self.timezone, self.latitude.reprAsLatitude(), self.longitude.reprAsLongitude())
	
	@property	
	def dataString(self):
		return "%s, %s - %s, %s)"%(self.countryName, self.timezone, self.latitude.reprAsLatitude(), self.longitude.reprAsLongitude())
		
	def __repr__(self):
		return "LocationEntry(%d, %s, %s)"%(self.id, self.name, self.country)


class LocationDict:

	def __init__(self, loclist):
		self.loclist = tuple(LocationEntry(loc) for loc in loclist)
	
	def __getitem__(self, idnumber):
		for loc in self.loclist:
			if loc.id == idnumber:
				return loc
		return None

	def getByName(self, name):
		return tuple(filter(lambda loc: loc.name == name, self.loclist))
	
	def listNames(self):
		return tuple(loc.name for loc in self.loclist)


class LocationOption(QWidget):

	locationChosen = pyqtSignal(int, str)

	def __init__(self, location):
		super().__init__()
		self.location = location
		layout = QHBoxLayout()
		self.label = QLabel()
		self.label.setText(self.location.labelString)
		layout.addWidget(self.label)
		self.submit = QPushButton(self.tr('select'))
		self.submit.clicked.connect(self._onbuttonclick)
		layout.addWidget(self.submit)
		self.setLayout(layout)

	def _onbuttonclick(self):
		self.locationChosen.emit(self.location.id, self.location.dataString)

class LocationSelector(QWidget):
	
	locationChosen = pyqtSignal(int, str)


	def __init__(self, locations, parent=None):
		super().__init__()
		
		layout = QGridLayout()
		self.options = tuple(LocationOption(location) for location in locations)
		for option in self.options:
			layout.addWidget(option)
			option.locationChosen.connect(lambda integer, datastring: self._onLocationChosen(integer, datastring))
		self.setLayout(layout)
		
	def _onLocationChosen(self, integer, datastring):
		self.locationChosen.emit(integer, datastring)
		self.close()


class LocationSearcher(QWidget):

	locationChosen = pyqtSignal(int)

	def __init__(self):
		super().__init__()

		layout = QVBoxLayout()
		self.searchbar = QLineEdit()
		self.searchbar.textEdited.connect(lambda text: self._searchInDatabase(text))
		self.completer = QCompleter([])
		self.completer.setCaseSensitivity(False)
		self.searchbar.setCompleter(self.completer)
		self.completer.activated[str].connect(self._submitSearch)
		self.dataLabel = QLabel()
		layout.addWidget(self.searchbar)
		layout.addWidget(self.dataLabel)
		self.setLayout(layout)
		self.selector = None #toplevel selector in case multiple locations correspond to the selected name
		self.locations = {}
		self.selectedId = None
		
	def _searchInDatabase(self, string):
		if len(string) == 3:
			locations = cruscoplanets.cmdline.searchLocation(string)
			self.locations = LocationDict(locations)
			self.completer.setModel(QStringListModel(self.locations.listNames(), self.completer))
	
	def _submitSearch(self, search):
		self.searchbar.setText(search)
		values = self.locations.getByName(search)
		if len(values) > 1:
			self.selector = LocationSelector(values)
			self.selector.show()
			self.selector.locationChosen.connect(lambda integer, datastring: self._onLocationChosen(integer, datastring))
		else:
			value = values[0]
			self.dataLabel.setText(value.dataString)
			self.selectedId = value.id
			loc_entry = LocationEntry(cruscoplanets.cmdline.searchLocationById(value.id, asDatabaseEntry = True))
			self.dataLabel.setText(loc_entry.dataString)
			self.locationChosen.emit(value.id)
	
	def _onLocationChosen(self, integer, datastring):
		self.selectedId = integer
		self.dataLabel.setText(datastring)
		self.locationChosen.emit(integer)
		
	def setLocation(self, location_id):
		self.selectedId = location_id
		location_object = LocationEntry(cruscoplanets.cmdline.searchLocationById(location_id, asDatabaseEntry = True))
		self.dataLabel.setText(location_object.dataString)
		self.searchbar.setText(location_object.name)
		
	def _closeLocationSelector(self):
		if self.selector != None:
			self.selector.close()
